/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviapartsviewrun.h"
#include <kdebug.h>
#include <klibloader.h>
#include <ktrader.h>


TaviaPartsViewRun::TaviaPartsViewRun(const KURL &url, const KParts::URLArgs &args, KService::Ptr serviceType, TaviaPartsView *pv, QWidget *window, bool removeReferrer, bool trustedSource, bool hideErrorDialog)
 : KParts::BrowserRun(url,args,pv->part,window,removeReferrer,trustedSource,hideErrorDialog)
{
    partView = pv;
    pURL = url;
    pArgs = args;
    TaviaPartsViewRun::serviceType = serviceType;
}


TaviaPartsViewRun::~TaviaPartsViewRun()
{
}




/*!
    \fn TaviaPartsViewRun::foundMimeType(const QString &_type)
 */
void TaviaPartsViewRun::foundMimeType(const QString &_type)
{
    mimetype = _type;
    KService::Ptr ptr;
    if (serviceType == 0L) {
        KTrader::OfferList offers = KTrader::self()->query(_type,
            "KParts/ReadOnlyPart",
            "and exist [Library]",
            QString::null);

        if (offers.count() == 0) {
            notFoundPartType(_type);
            return;
        }
        ptr = offers.first();
        kdDebug() << "PartName: " << ptr->name() << endl;
    } else {
        ptr = serviceType;
    }
    if (ptr->name() == "KHTML") {
        if (partView->service != 0) {
            partView->manager->getPartManager()->removePart(partView->part);
            delete partView->part;
            partView->service = 0L;
            partView->part = partView->htmlPart;
            partView->htmlPart->widget()->show();
            emit partView->activate();
        }
        partView->htmlPart->browserExtension()->setURLArgs(pArgs);
        partView->loadingStarted();
        partView->htmlPart->openURL(pURL);
        return;
    }
    if (partView->service != 0) {
        if (ptr->name() == partView->service->name()) {
            KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(partView->part);
            if (ext != 0) {
                ext->setURLArgs(pArgs);
            }
            partView->loadingStarted();
            partView->part->openURL(pURL);
            return;
        } else {
            partView->manager->getPartManager()->removePart(partView->part);
            delete partView->part;
        }
    }
    KParts::ReadOnlyPart *part = partView->loadParts(ptr);
    if (part) {
        partView->htmlPart->widget()->hide();
        partView->htmlPart->closeURL();
        partView->part = part;
        KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(part);
        if (ext != 0L) {
            ext->setURLArgs(pArgs);
        }
        part->widget()->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
        part->embed(partView->frame);
        partView->frameLay->addWidget(part->widget());
        partView->loadingStarted();
        part->openURL(pURL);
        partView->manager->getPartManager()->addPart(part,partView->viewActive);
                
    }

}


/*!
    \fn TaviaPartsViewRun::scanFile()
 */
void TaviaPartsViewRun::scanFile()
{
    KParts::BrowserRun::scanFile();
    KIO::TransferJob *job = dynamic_cast<KIO::TransferJob*>(m_job);
    if (job && !job->error()) {
        connect(job,SIGNAL(redirection(KIO::Job *,const KURL&)),
            SLOT(slotRedirection(KIO::Job *,const KURL&)));
    }

}


/*!
    \fn TaviaPartsViewRun::slotRedirection(KIO::Job *job, const KURL& toURL)
 */
void TaviaPartsViewRun::slotRedirection(KIO::Job *job, const KURL& toURL)
{
    // from KonqRun
    KURL fromURL = static_cast<KIO::TransferJob *>(job)->url();
    if (fromURL != toURL) {
        m_args.setDoPost(false);
        pArgs.setDoPost(false);
        m_args.setRedirectedRequest(true);
        pArgs.setRedirectedRequest(true);
        pURL = toURL;
    }
}


/*!
    \fn TaviaPartsViewRun::handleError (KIO::Job *job)
 */
void TaviaPartsViewRun::handleError (KIO::Job *job)
{
    BrowserRun::handleError(job);
}


/*!
    \fn TaviaPartsViewRun::notFoundPartType(const QString &mimetype)
 */
void TaviaPartsViewRun::notFoundPartType(const QString &mimetype)
{
    KParts::BrowserRun::AskSaveResult res = askSave(pURL,0L,mimetype);
    if (res == KParts::BrowserRun::Save) KParts::BrowserRun::save(pURL,QString::null);
    if (res == KParts::BrowserRun::Open) KRun::displayOpenWithDialog(pURL,false);
}





