/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TAVIAVIEWMANAGER_H
#define TAVIAVIEWMANAGER_H

#include <qobject.h>
#include <qptrlist.h>
#include <qmap.h>
#include <kmdichildview.h>
#include <kparts/partmanager.h>
#include <kxmlguiclient.h>
#include <kaction.h>
#include <taviamdifrm.h>
#include <taviaview.h>
#include <taviatabbox.h>
#include <taviatabbutton.h>
#include <taviaurcmanager.h>

/**
@author TAM(Teppei Tamra)
*/
class TaviaViewEntry
{
public:
    TaviaView *view;
    TaviaTabButton *button;
    KMdiChildView *childView;
    QString caption;
};

class TaviaViewManager : public QObject , public KXMLGUIClient
{
Q_OBJECT
public:
    TaviaViewManager(TaviaTabBox *_tabbox, TaviaMdiFrm *_mdi ,QObject *parent = 0,const char *name = 0);

    ~TaviaViewManager();
    TaviaTabBox * getTabBox();
    TaviaMdiFrm * getMdiFrm();
    KParts::PartManager * getPartManager();
    TaviaURCManager * getURCManager();
    bool isSelected(TaviaView *view);

public:
    bool closeProcess;

protected:
    QPtrList<TaviaViewEntry> viewList;
    TaviaTabBox *tabbox;
    TaviaMdiFrm *mdi;
    KParts::PartManager *partManager;
    TaviaViewEntry * currentView;
    TaviaURCManager *urcManager;

protected:
    TaviaViewEntry * entrySearch(TaviaTabButton *e);
    TaviaViewEntry * entrySearch(TaviaView *e);
    TaviaViewEntry * entrySearch(KMdiChildView *e);
    TaviaViewEntry * addView(TaviaView *view);
    void delView(TaviaViewEntry *entry);
public slots:
    void addPartsView(const KURL &url = KURL(),const KParts::URLArgs &args = KParts::URLArgs());
    void addPartsView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p);
    void setSelected(unsigned int s);
    void closeView(TaviaTabButton *b);
    void closeView(TaviaView *v);
    void closeView(KMdiChildView *cv);
    void openURLwithCurrentView(const KURL &url,const KParts::URLArgs &args = KParts::URLArgs());
protected slots:
    void updateView();
    void setTitleText(const QString &title);
    void setStatusText(const QString &text);
    void setIcon(const QPixmap &icon);
    void setSelected(TaviaTabButton *b);
    void setSelected(TaviaView *v);
    void setSelected(TaviaViewEntry *e);
    void setSelected(KMdiChildView *cv);
signals:
    void activePart(KParts::Part *part);
};

#endif
